'use strict';

module.exports = function moduleExports(grunt) {
    // Load all grunt tasks
    require('load-grunt-tasks')(grunt, {
        pattern: ['grunt-*', 'FET-grunt-*']
    });

    // Measure the time for the tasks
    require('time-grunt')(grunt);

    // Project configuration.
    grunt.initConfig({

        pkg: grunt.file.readJSON('package.json'),
        cfg: grunt.file.readJSON('config.json'),

        // https://github.com/addyosmani/grunt-uncss
        uncss: {
            dist: {
                options: {
                    ignore: [
                        // /(\.is-){1}(?!(active|other))(\w)+/g,
                        // /(\.has-){1}(?!(states|other))(\w)+/g,
                        // /\.js-/g
                    ]
                },
                files: [{
                    nonull: true,
                    src: ['http://fed.cxo.name:8080/'],
                    dest: 'dist/css/uncss-styleguide.css'
                }]
            }
        },

        // bower
        bower: {
            install: {
                options: {
                    layout: 'byComponent',
                    targetDir: 'lib',
                    verbose: true
                }
            }
        },


        // favicon (npmjs.com/package/grunt-real-favicon)
        realFavicon: {
            myIcon: {
                src: '<%= cfg.src %>/src-favicon/favicon.png',
                dest: '<%= cfg.src %>/stock/favicon',
                options: {
                    iconsPath: '##BASEURL##/##VERSION##/favicon/',
                    html: ['<%= cfg.src %>/favicon_ssi.html'],
                    design: {
                        desktop_browser: {},
                        ios: {
                            picture_aspect: 'no_change'
                        },
                        windows: {
                            picture_aspect: 'no_change',
                            background_color: '#eb443f'
                        },
                        android_chrome: {
                            picture_aspect: 'no_change',
                            manifest: {
                                name: 'Chip',
                                display: 'standalone',
                                orientation: 'portrait',
                                start_url: ''
                            },
                            theme_color: '#eb443f'
                        }
                    },
                    settings: {
                        compression: 2
                    }
                }
            }
        },


        // sass
        sass: {
            options: {
                sourceMap: true,
                includePaths: [
                    '<%= cfg.assets %>/scss',
                    '<%= cfg.src %>/scss'
                ]
            },
            run: {
                files: {
                    '<%= cfg.assets %>/css/prototype.css': '<%= cfg.assets %>/scss/prototype.scss',
                    '<%= cfg.assets %>/css/styleguide.css': '<%= cfg.assets %>/scss/styleguide.scss',
                    '<%= cfg.assets %>/css/widgets.css': '<%= cfg.assets %>/scss/widgets.scss'
                }
            }
        },


        // cssnano
        cssnano: {
            options: {
                safe: true,
                sourcemap: false,
                autoprefixer: {
                    add: true,
                    browsers: ['last 2 versions', 'ie 9']
                }
            },
            build: {
                options: {
                    calc: false, colormin: false, convertValues: false, core: false, discardComments: false, discardDuplicates: false, discardEmpty: false, discardUnused: false, filterOptimiser: false, filterPlugins: false, functionOptimiser: false, mergeIdents: false, mergeLonghand: false, mergeRules: false, minifyFontValues: false, minifyGradients: false, minifySelectors: false, normalizeCharset: false, normalizeUrl: false, orderedValues: false, reduceIdents: false, reduceTransforms: false, styleCache: false, svgo: false, uniqueSelectors: false, zindex: false // eslint-disable-line
                },
                files: {
                    'assets/css/prototype.css': 'assets/css/prototype.css',
                    'assets/css/styleguide.css': 'assets/css/styleguide.css',
                    'assets/css/widgets.css': 'assets/css/widgets.css'
                }
            }
        },


        // run commands
        run: {
            server: {
                args: ['./node_modules/sourcejs/app.js'],
                options: {
                    passArgs: [
                        'port'
                    ]
                }
            }
        },


        // watch
        watch: {
            scss: {
                options: {
                    livereload: true
                },
                files: [
                    '<%= cfg.assets %>/scss/**/*.scss',
                    '<%= cfg.src %>/scss/**/*.scss'
                ],
                tasks: ['build']
            }
        },


        // clean
        clean: {
            assets: ['<%= cfg.assets %>/css']
        },


        // stylelint
        stylelint: {
            options: {
                configFile: '.stylelintrc'
            },
            src: [
                '<%= cfg.src %>/scss/**/*.scss'
            ]
        },


        // concurrent
        concurrent: {
            startServerAndWatcher: {
                tasks: ['run:server', 'watch:scss'],
                options: {
                    logConcurrentOutput: true
                }
            }
        }
    });


    // install task
    grunt.registerTask('install', [
        'bower',
        'build'
    ]);

    // run build
    grunt.registerTask('build', [
        'clean:assets',
        'stylelint',
        'sass',
        'cssnano'
    ]);

    // run server and watcher
    grunt.registerTask('server', [
        'concurrent:startServerAndWatcher'
    ]);

    // create tag
    grunt.registerTask('tag', [
        'stylelint',
        'FET-grunt-tag',
        'build'
    ]);

    // favicon Task
    grunt.registerTask('favicon', ['realFavicon']);
};
