define( [ 'jquery', 'underscore', 'component/core/mediator', 'viewportService' ],
  function( $, _, mediator, viewportService ) {

    'use strict';

    var defaults = {
      scrollThreshold: 1500,
      title: 'Video on Demand Services',
      link: '',
      url: 'http://fed.cxo.name:9001/data/vetalio.jsonp',
      redirectUrl: 'http://x.chip.de/linktrack/cloudsider/',
      containerId: '',
      pageType: chip_currpage_json.bart_name || ''
    };

    function Vetalio( selector ) {
      this.$selector = $( selector );

      if ( !( this instanceof Vetalio ) ) {
        return new Vetalio();
      }

      if ( !this.$selector.data( 'Vetalio' ) ) {
        this.init();
      }
    }

    Vetalio.prototype.init = function() {
      this.$selector.data( 'Vetalio', this );
      this.params = this.$selector.data( 'vetalio' );
      this.params = _( this.params ).isObject() ? this.params : {};
      this.settings = _( this.params).defaults( defaults );
      this.key = this.settings.key || _( 'Vetalio_' ).uniqueId();

      this.template = _( this.widgetTemplate() ).template();
      
      this.$widget = this.$selector;

      //this.bindEvents();
      this.loadData();
      this.subscribeToViewportService();
    };

    Vetalio.prototype.widgetTemplate = function() {
      var template = [
        '<header>',
          '<h3 class="VetalioWidget__Header"><%= title %></h3>',
          '<small><%= subtitle %></small>',
        '</header>',
        '<% _.each(items, function( item ) { %>',
          '<article>',
            '<div class="VetalioWidget__ProviderLogo">',
              '<img alt="<%= item.name %>" src="<%= item.logo %>">',
            '</div>',
            '<div class="VetalioWidget__Price">',
              '<%= item.price %> &euro;',
              '<small><%= item.chip_bezugsgroesse %></small>',
            '</div>',
            '<a class="VetalioWidget__ProviderLink" href="<%= item.providerLink %>" target="_blank">',
              '<span>Zum Anbieter</span>',
            '</a>',
            '<% if ( item.aktionshinweis ) { %>',
              '<div class="VetalioWidget__Feature">',
                '<span><%= item.aktionshinweis %></span>',
              '</div>',
            '<% } %>',
            '<ul class="VetalioWidget__FactList">',
            '<% _.each(item.pro, function( pro ) { %>',
              '<li class="VetalioWidget__Fact VetalioWidget__Fact--Pro">',
                '<%= pro %>',
              '</li>',
            '<% }); %>',
            '<% _.each(item.con, function( con ) { %>',
              '<li class="VetalioWidget__Fact VetalioWidget__Fact--Pro">',
                '<%= con %>',
              '</li>',
            '<% }); %>',
            '</ul>',
            '<div class="VetalioWidget__Rating">',
              '<% for( var i = 0; i < 5; i++ ) { %>',
                '<div class="VetalioWidget__RatingStar<% if (item.rating.stars > i){ %> is-active<% }%>"></div>',
              '<% } %>',
              '<div class="VetalioWidget__RatingValue">',
                '<%= item.rating.grade %> (<%= item.rating.score %>%)',
              '</div>',
              '<a class="VetalioWidget__Link" href="<%= item.detailLink %>">→ mehr Details</a>',
            '</article>',
        '<% }); %>'];
      return template.join( '' );
    };

    Vetalio.prototype.subscribeToViewportService = function() {
      mediator.subscribe( 'change' , $.proxy( this.loadData, this ), 'viewportService' );
    };

    Vetalio.prototype.loadData = function() {
      var viewport = viewportService.getViewport(),
        widgetOffsetTop = this.$widget.offset().top,
        loadWidget = viewport.scrollOffsetTop > ( widgetOffsetTop - this.settings.scrollThreshold );

      if ( !loadWidget || this.$widget.hasClass( 'is-loaded' ) ) {
        return;
      }

      this.$widget
        .addClass( 'is-loaded' );

      $.ajax( {
        type: 'GET',
        url: this.settings.url,
        async: false,
        jsonpCallback: 'jsonCallback',
        contentType: 'application/json',
        dataType: 'jsonp',
        success: $.proxy( this.renderWidget, this )
      } );
    };

    Vetalio.prototype.renderRating = function( score ) {
      var grade = 'Schlecht',
        stars = 0,
        grades = {
          80: 'Sehr Gut',
          60: 'Gut',
          40: 'Befriedigend',
          20: 'Ausreichend',
          0: 'Schlecht'
      };

      _( grades ).each( function( schoolGrade, value ) { 
        if ( score > parseInt( value ) ) {
          stars++;
          grade = schoolGrade;
        }
      } );

      return {
        stars: stars,
        grade: grade,
        score: score
      };
    };

    Vetalio.prototype.renderWidget = function( result ) {
      var items = result.vod; // temporary untill we have single jsonps

      _( items ).each( $.proxy( function( item, index ) {
          items[ index ].rating = this.renderRating( item.score_overall );
          items[ index ].providerLink = [ this.settings.redirectUrl,
            'providerinfo/?url=', item.affiliate_link,
            '&container_id=', window.chip_currpage_json.container_id,
            '&pagetitle=', window.chip_currpage_json.pagetitle, 
            '&provider=', item.name,
            '&=pagetype=', window.chip_currpage_json.bart_name ].join( '' );

          items[ index ].detailLink = '#';
      }, this ) );

      var html = this.template( {
        title: this.settings.title,
        subtitle: 'Partnerinhalt von <a class="VetalioWidget__Link" href="' + this.settings.link +'" target="_blank" rel="nofollow">vetalio</a>',
        items: items
      } );

      this.$widget
        .append( html );
    };

    // Initialization happens when **mediator** publishes the event `ui.init`.
    mediator.subscribe( 'ui.init', function() {
      $( '[data-vetalio]' ).each( function() {
        new Vetalio( this );
      } );
    } );

    return Vetalio;
  } );