define([ 'jquery', 'underscore', 'component/core/mediator', 'raven' ],
function( $, _, mediator, Raven ) {

    'use strict';

    var defaults = {
    }, component = {
            name: 'WidgetEnergy',
            slug: 'widget-energy'
    };

    function WidgetEnergy( selector ) {
        this.$selector = $(selector);

        if ( !this.$selector.data( component.name ) ) {
            try {
                this.init();
            } catch( err ) {
                Raven.captureException( err, {
                    extra: {
                        module: component.name
                    }
                });
            }
        }
    }

    WidgetEnergy.prototype.init = function() {
        var args        = this.$selector.data( component.slug ),
            params      = _( args ).isObject() ? args : {};

        this.settings   = _( params ).defaults( defaults );
        this.key        = this.settings.key || _( component.name + '_' ).uniqueId();
        this.activeForm = 0;
        this.$tabs      = $( '.WidgetEnergy__Tabs a', this.$selector );
        this.$forms     = $( '.WidgetEnergy__Form', this.$selector );
        this.$radios    = $( '.WidgetEnergy__Persons label, .WidgetEnergy__House label', this.$selector );

        this.bindEvents();
        this.$selector.data( component.name, this );
    };

    WidgetEnergy.prototype.bindEvents = function() {
        this.$tabs
            .on( 'touchstart.' + this.key, $.proxy( this.switchTab, this ) )
            .on( 'click.' + this.key, $.proxy( this.switchTab, this ) );

        this.$radios
            .on( 'touchstart.' + this.key, $.proxy( this.updateConsumptionValue, this ) )
            .on( 'click.' + this.key, $.proxy( this.updateConsumptionValue, this ) );
    };

    WidgetEnergy.prototype.updateConsumptionValue = function( event ) { 
        var $label = $( event.target ).closest( 'label' ),
            $radio = $('#' + $label.attr('for'));

        $radio
            .closest( 'form')
            .find('.WidgetEnergy__Consumption input' )
            .val( $radio.val() );
    };

    WidgetEnergy.prototype.switchTab = function( event ) {
        var $tab = $( event.target ).closest( '.WidgetEnergy__Tabs a' );

        this.$tabs
            .removeClass( 'is-active' );

        $tab
            .addClass( 'is-active' );

        this.$forms
            .removeClass( 'is-visible' )
            .filter( '[id=' + $tab.attr( 'href' ).replace( '#', '' ) +']' )
            .addClass( 'is-visible' );

        event.preventDefault();
        event.stopPropagation();
    };

    mediator.subscribe( 'ui.init', function() {
      $( '[data-' + component.slug + ']' ).each( function() {
        new WidgetEnergy( this );
      } );
    } );

    return WidgetEnergy;
});