define([ 'jquery', 'underscore', 'component/core/mediator', 'raven' ],
function( $, _, mediator, Raven ) {

    'use strict';

    var defaults = {
        templateId: '#tpl-widgetNewsletter',
        url: 'http://www.chip.de/newsletter.subscription.widget/actions/register.php?ajax=true&jsonp=true',
        successMessage: 'Vielen Dank! Sie erhalten in Kürze eine E-Mail zur Bestätigung ihrer Anmeldung.'
    }, component = {
        name: 'WidgetNewsletter',
        slug: 'widget-newsletter',
        channel: 'ui.widget-newsletter'
    };

    function WidgetNewsletter( selector ) {
        this.$selector = $( selector );

        if ( !this.$selector.data( component.name ) ) {
            try {
                this.init();
            } catch( err ) {
                Raven.captureException( err, {
                    extra: {
                        module: component.name
                    }
                });
            }
        }
    }

    WidgetNewsletter.prototype.init = function() {
        var args       = this.$selector.data( component.slug ),
            params     = _( args ).isObject() ? args : {};

        this.settings  = _.defaults( params, defaults );
        this.key       = this.settings.key || _( component.name ).uniqueId();
        this.template  = _.template( $( this.settings.templateId ).html() );
        this.$widget   = this.$selector;
        this.$nId      = $( '.' + component.name + '__NewsletterId', this.$widget );
        this.$form     = $( '.' + component.name + '__Form', this.$widget );
        this.$checkbox = $( '.' + component.name + '__Checkbox input', this.$widget );
        this.$email    = $( '.' + component.name + '__Email', this.$widget );

        this.bindEvents();
        this.$selector.data( component.name, this);
    };

    WidgetNewsletter.prototype.bindEvents = function() {
        this.$form.on( 'submit.' + this.key, $.proxy( this.submitForm, this ) );
    };

    WidgetNewsletter.prototype.submitForm = function( event ) {
        var url = [ this.settings.url, 
            '&nl_mail=', this.$email.val(),
            '&nl_id=', this.$nId.val(),
            '&nl_optin=', this.$checkbox.is(':checked') ? 1 : 0
        ]

        event.preventDefault();
        event.stopPropagation();

        $.ajax({
            type: 'GET',
            url: url.join( '' ),
            async: false,
            jsonpCallback: 'jsonCallback',
            contentType: 'application/json',
            dataType: 'jsonp',
            success: $.proxy( this.handleResponse, this ),
            error: $.proxy( this.handleResponse, this )
        });
    };

    WidgetNewsletter.prototype.handleResponse = function( data ) {
        this.hideMessage();

        if ( data.status === 'success' ) {
            this.showMessage( 'Success', this.settings.successMessage );
        } else {
            this.showMessage( 'Warning', data.error.message );
        }
    };

    WidgetNewsletter.prototype.hideMessage = function( ) {
        this.$widget.find('.' + component.name + '__Message' ).remove();
    };

    WidgetNewsletter.prototype.showMessage = function( state, message ) {
        var html = this.template({
            state: state,
            message: message,
        });
     
        this.$email.after( html );
        
        //@TODO this should be moved out of the component into a tracking service, to work
        // with dtm, old tracking etc.
        mediator.publish( 'newsletter.on.submit.' + state.toLowerCase(),  this.settings.nId );
    };

    mediator.subscribe( 'ui.init', function() {
        $( '[data-' + component.slug + ']' ).each( function() {
            new WidgetNewsletter( this );
        } );
    } );
   
    return WidgetNewsletter;
});