define([ 'jquery', 'underscore', 'component/core/mediator', 'viewportService', 'raven' ],
function( $, _, mediator, viewportService, Raven ) {

    'use strict';

    var defaults = {
        scrollThreshold: viewportService.getViewport().viewportHeight * 1.5,
        title: false,
        permalink: window.location.href,
        footer: false,
        templateId: '#tpl-widgetOutbrain',
        tags: []
    }, component = {
        name: 'WidgetOutbrain',
        slug: 'widget-outbrain'
    };

    function WidgetOutbrain( selector ) {
        this.$selector = $( selector );

        if ( !this.$selector.data( component.name ) ) {
            try {
                this.init();
            } catch( err ) {
                Raven.captureException( err, {
                    extra: {
                        module: component.name
                    }
                });
            }
        }
    }

    WidgetOutbrain.prototype.init = function() {
        var args             = this.$selector.data( component.slug ),
            params           = _( args ).isObject() ? args : {};

        this.settings        = _( params ).defaults( defaults );
        this.key             = this.settings.key || _( component.name + '_' ).uniqueId();
        this.recommendations = [];
        this.counter         = 0;
        this.template        = _.template( $( this.settings.templateId ).html() );
        this.$widget         = this.$selector;

        this.bindEvents();
        this.loadRecommendations();
        this.subscribeToViewportService();
        this.$selector.data( component.name, this );
    };

    WidgetOutbrain.prototype.bindEvents = function() {
        this.$widget
            .on( 'touchstart.' + this.key, 'a', $.proxy( this.redirectUrl, this ) )
            .on( 'mousedown.' + this.key, 'a', $.proxy( this.redirectUrl, this ) );
    };

    WidgetOutbrain.prototype.redirectUrl = function( event ) {
        var $link = $( event.target ).closest( 'a' );

        $link
            .attr( 'href', $link.data( 'widget-outbrain-redirect' ) )
            .removeAttr( 'data-widget-outbrain-redirect' );
    };

    WidgetOutbrain.prototype.subscribeToViewportService = function() {
        mediator.subscribe( 'scroll' , $.proxy( this.loadRecommendations, this ), 'viewportService' );
    };

    WidgetOutbrain.prototype.addRecommendation = function( result ) {
        this.recommendations = this.recommendations.concat( result.doc );
        this.counter = this.counter + 1;

        if ( this.counter === this.settings.tags.length ) {
            this.renderWidget( this.recommendations );
        }
    };

    WidgetOutbrain.prototype.loadRecommendations = function() {
        var permalink = this.settings.permalink,
            viewport = viewportService.getViewport(),
            widgetOffsetTop = this.$widget.offset().top,
            loadWidget = viewport.scrollOffsetTop > ( widgetOffsetTop - this.settings.scrollThreshold );

        if ( !window.OBR || !loadWidget || this.$widget.hasClass( 'is-loaded' ) ) {
            return;
        }

        this.$widget.addClass( 'is-loaded' );

        _( this.settings.tags ).each( $.proxy( function( tag ) {
            window.OBR.extern.callRecs( {
                permalink: permalink,
                widgetId: tag
            }, $.proxy( this.addRecommendation, this ) );
        }), this );
    };

    WidgetOutbrain.prototype.renderWidget = function( recommendations ) {

      var html = this.template( {
        title: this.settings.title,
        items: recommendations,
        footer: this.settings.footer
      } );

      this.$widget.append( html );
    };

    mediator.subscribe( 'ui.init', function() {
      $( '[data-' + component.slug + ']' ).each( function() {
        new WidgetOutbrain( this );
      } );
    } );

    return WidgetOutbrain;
} );