define(
    ['require', 'jquery', 'underscore', 'mediator', 'viewportService', 'raven'], function (req, $, _, Mediator, viewportService, Raven) {
        'use strict';

        var defaults = {
            'namespace': 'searchahead',
            'class_loading': 'is-loading',
            'class_selected': 'is-selected',
            'class_visible': 'is-visible',
            'keystroke_delay': 250,
            'maximum_suggestions': 5,
            'minimum_input': 1,
            'selector_parent_element': 'form',
            'template': '#tpl-searchflyout',
            'suggestList': 'products',
            'suggestNames': 'Top Produkte',
            'searchbooster': '',
            'autocomplete': '',
            'autosuggest': ''
        };

        /**
         * The Searchahead object
         *
         * @param selector
         * @param options
         * @returns {Searchahead}
         * @constructor
         */
        function Searchahead(selector) {
            this.$selector = $(selector).eq(0);
            if ( !(this instanceof Searchahead) ) {
                return new Searchahead();
            }

            if ( !this.$selector.data('Searchahead') ) {
                try {

                    this.init();
                } catch (err) {
                    Raven.captureException(err, {
                        extra: {
                            'module': 'Searchahead'
                        }
                    });
                }
            }
        };

        Searchahead.prototype.init = function () {
            this.timeout = null;

            // setting the parameter for this Element otherwise take the fallback parameter
            this.$selector.data('Searchahead', this);
            this.params = this.$selector.data('searchahead');
            this.params = _(this.params).isObject() ? this.params : {};
            this.settings = _(this.params).defaults(defaults);

            this.currentViewport = viewportService.getViewport();
            this.$parent = this.$selector.closest(this.settings.selector_parent_element);
            this.sInput = this.$selector.val();
            this.$results = null;
            this.lock = false;

            this.activateListeners();

        };
        /**
         * start ajax-Request fetch the autocomplete-words and optional the suggestions' list
         */
        Searchahead.prototype.getResults = function () {
            var _this = this,
                Results = [];

            _this.$selector.addClass(_this.settings.class_loading);
            $.ajax({
                url: _this.settings.autocomplete + '?q=' + (_this.sInput).toLowerCase() + _this.settings.searchbooster + '&callback=autocomplete',
                dataType: 'jsonp',
                jsonp: false,
                jsonpCallback: 'autocomplete' || 'define',
                success: function (data) {
                    var hits = [];
                    $.each(data.hits, function (key, val) {
                        // search for value and replace it
                        var searchhit = data.hits[key].search;
                        hits.push({
                            'value': searchhit,
                            'highlight': searchhit.replace((_this.sInput).toLowerCase(), '<strong>' + (_this.sInput).toLowerCase() + '</strong>')
                        });
                    });

                    Results['complete'] = hits;
                    if ( !(_this.settings.autosuggest) || _this.settings.autosuggest !== '' ) {
                        setTimeout(function () {
                            _this.parseSuggestions(Results);
                        }, 250);
                    }

                },
                error: function (data, err) {
                    Raven.captureException(err);
                }
            });

            if ( _this.settings.autosuggest != '' ) {
                $.ajax({
                    url: _this.settings.autosuggest + '?q=' + _this.sInput + _this.settings.searchbooster + '&callback=autosuggest',
                    dataType: 'jsonp',
                    jsonp: false,
                    jsonpCallback: 'autosuggest' || 'define',
                    success: function (data) {
                        $.extend(Results, data)
                        _.delay(function () {
                            $.proxy(_this.parseSuggestions, _this, Results);
                        }, 250);

                    },
                    error: function (data, err) {
                        Raven.captureException(err);
                    }
                });
            }

            return true;
        }

        Searchahead.prototype.activateListeners = function () {
            var _this = this;

            $(document)
            // close on click outside of the autosuggest
                .on('click.' + _this.settings.namespace + ' touchend.' + _this.settings.namespace, function (e) {
                    var $target = $($(e.target));
                    // has the click occured outside of the autosuggest, that means result list and input field
                    if ( !!_this.$results && _this.$results.has(e.target).length === 0 && _this.$selector[0] !== e.target ) {
                        _this.toggleResults('hide');
                    } else if ( _this.copyItemValue($target) ) {
                        $target.addClass(_this.settings.class_selected);
                         if (_this.currentViewport.viewportRange != 'small') {
                                _this.$selector.closest('form').submit();
                            }
                    }
                })
                // directional keystrokes to navigate through the results
                .on('keydown.' + _this.settings.namespace, function (e) {
                    var keyCode = e.keyCode,
                        myTarget = null;
                    //'\r'.charCodeAt(0);
                    /* left = 37; up = 38; right = 39; down = 40; tab = 9; enter = 13|108 */
                    switch (keyCode) {
                        case 37:
                        case 38:
                            _this.navigateItem('prev');
                            break;
                        case 39:
                        case 40:
                            _this.navigateItem('next');
                            break;
                        case 9:
                        case 13:
                        case 108:
                            // tab => copy selected to input field
                            // enter => copy selected to input field & submit
                            myTarget = $('.SearchResults--AutoComplete li.' + _this.settings.class_selected + ' a', _this.$results);

                            _this.copyItemValue(myTarget);
                            break;
                        default:
                            _this.$parent.find('input[name=it]').val('1');
                    }
                });

            /*_this.$results
             .find('.SearchBoxAutoComplete__Suggest li')
             .on('mouseover.'+_this.settings.namespace, function (e) {
             var $target = $(e.currentTarget);
             $('.' + _this.settings.class_selected, _this.$results).removeClass(_this.settings.class_selected);
             $target.addClass(_this.settings.class_selected);
             });*/
            // TODO: problems with focus and blur
            this.$selector.on('focus.autosuggest', function (e) {
                // shortly after the focus event there is a click event. this code kills that ... once
                _this.$selector.on('click.autosuggest.onfocus', function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    _this.$selector.off('click.autosuggest.onfocus');
                    return false;
                });

                // is the current text long enough
                if ( _this.isValidInput(self) ) {

                    // if the list is already present
                    if ( _this.$results ) {
                        // show the list
                        _this.toggleResults('show');
                    } else {
                        // get the list
                        _this.getResults();
                    }
                }

                // start listening for changes
                // more reliable to listen with an interval than to try and guess all of the different methods for
                // changing the input's value, ie. keyup, paste, delete, etc.
                _this.startInterval(_this.settings.keystroke_delay);

            })
                .on('blur.autosuggest', function (e) {
                    // stop listening for changes
                    _this.stopInterval();
                });

        };

        Searchahead.prototype.deactivateListeners = function () {
            this.$results
                .find('.SearchResults li')
                .off('mouseover.' + this.settings.namespace);
            $(document)
                .off(
                    'click.' + this.settings.namespace +
                    ' touchend.' + this.settings.namespace
                )
                .off('keydown.' + this.settings.namespace);
        };

        /**
         * Move the selection to the next item in Direction in the results' list
         */
        Searchahead.prototype.navigateItem = function (direction) {
            var _this = this,
                $items = $('.SearchResults li', _this.$results),
                $selected = $items.filter('.' + _this.settings.class_selected),
                $new = null,
                selectedIndex = $items.index($selected),
                newIndex = 0,
                step = 0;

            if ( _this.$results && _this.$results.hasClass(_this.settings.class_visible) ) {
                if ( direction === 'prev' ) {
                    step = -1;
                } else {
                    step = 1;
                }
                newIndex = selectedIndex + step;
                if ( newIndex < 0 ) {
                    newIndex = $items.length - 1;
                }
                else if ( newIndex > $items.length - 1 ) {
                    newIndex = 0;
                }

                $selected.removeClass(_this.settings.class_selected);

                $($items[newIndex]).addClass(_this.settings.class_selected);
            }
        };

        /**
         * For AutoCompletion Suggestions copys the Value to the Searchfield
         */
        Searchahead.prototype.copyItemValue = function (target) {
            var _this = this,
                completevalue = target.data('completeValue') || '';

            if ( completevalue != '' ) {
                // TODO: html() will only work if there is nothing but text in the list item element
                _this.$selector.val(completevalue);
                _this.$parent.find('input[name=it]').val('2');
                _this.throwEvent('autocomplete:selected:' + completevalue);
                return true;
            }
            return false;
        }

        /**
         * Remove the auto-suggest result list from view and remove all events associated with it
         *
         * @param oAutoSuggest
         * @returns {*}
         */
        Searchahead.prototype.toggleResults = function (state) {
            var _this = this;
            if ( !_this.$results ) {
                return false;
            }
            _this.deactivateListeners();
            if ( state === 'hide' ) {
                _this.$results
                    .removeClass(_this.settings.class_visible)
                    .find('.SearchResults li')

            } else {
                _this.$results
                    .addClass(_this.settings.class_visible);
                if ( _this.currentViewport.viewportRange === 'small' ) {
                    _this.$results
                        .find('.SearchFlyout')
                        .css({
                            'max-height': (_this.currentViewport.viewportHeight - 100) + 'px',
                            'overflow-y': 'scroll'
                        });
                } else {
                    _this.$results
                        .find('.SearchFlyout')
                        .removeAttr('style');
                }
                _this.lock = true;
                _this.throwEvent('update' + _this.$selector.id);
                _this.activateListeners();
            }

            return _this.$results;
        };

        // ##### throwEvent()
        //
        //
        //
        Searchahead.prototype.throwEvent = function (message) {
            var evt,
                detail = {detail: {message: message}};
            if ( typeof window.CustomEvent === 'function' ) {
                evt = new CustomEvent(this.settings.namespace, detail);
            } else {
                evt = document.createEvent('CustomEvent');
                evt.initCustomEvent(this.settings.namespace, false, false, detail);
            }
            window.dispatchEvent(evt);
            //console.log(this.settings.namespace+'//'+message+'//');
        };

        Searchahead.prototype.parseSuggestions = function (data) {
            var _this = this;
            var template,
            //tmpl_string,
                sInput = data.input || '',
                i, sFormUrl = _this.$selector.closest('form').attr('action') + '?';

            //set the name of the data parsed to the template to data (makes things easier)
            _.templateSettings.variable = "data";
            //fetch our html for the template from external file
            template = _.template(
                $(this.settings.template).html()
            );

            if ( typeof(data['listname']) !== 'undefined' ) {
                //cut down suggestions to the max. displayable Suggestions
                data['listname'] = data['listname'].slice(0, _this.settings.maximum_suggestions);

                // create url and add 'it' values to the end of each item's link
                for ( i = data['listname'].length; i > 0; i-- ) {
                    data['listname'][i]['url'] = sFormUrl + 'q=' + data['listname'][i].tokenname;
                }
            }

            if ( (_this.settings.autosuggest) || _this.settings.autosuggest !== '' ) {
                data['categories'] = _this.settings.suggestList.split(',');
                data['categorienames'] = _this.settings.suggestNames.split(',');

            }

            _this.$selector.removeClass(_this.settings.class_loading);

            if ( data['complete'] !== null || _this.settings.autosuggest ) {
                _this.addResultsMarkup(template(data));
                _this.toggleResults('show');
            }
        };

        /**
         * Add the new list to the page and the AutoSuggest object
         *
         * @param $results
         * @returns {replaceAll|*|replaceAll|null|$results|$results}
         */
        Searchahead.prototype.addResultsMarkup = function (results) {
            var _this = this;
            var $results = $($.parseHTML(results));

            if ( _this.$results ) {
                _this.$results.remove();
            }
            $results.insertAfter(_this.$parent);
            return _this.$results = $results;
        }

        /**
         * Check if the length of the input string is sufficient
         *
         * @param oSearchahead
         * @returns {boolean}
         */
        Searchahead.prototype.isValidInput = function () {
            var _this = this;
            // is the input sufficient to warrant making a request
            return (_this.sInput.length >= _this.settings.minimum_input);
        };

        /**
         * Check if the current text in the input field is different than what was there last time it was checked
         *
         * @param oAutoSuggest
         * @returns {boolean}
         */
        Searchahead.prototype.isDifferentInput = function () {
            var sInput = this.$selector.val();
            return (this.sInput !== sInput);
        };

        // TODO: slow down interval incrementally until a point at which the input truly changes
        Searchahead.prototype.startInterval = function (nIntervalMS) {
            var nIterations = 0;

            this.timeout = setInterval(function () {
                var sInput = this.$selector.val();

                // is the current text different than in the previous iteration
                if ( this.isDifferentInput() ) {
                    this.sInput = sInput;

                    // is the current text long enough
                    if ( this.isValidInput() ) {
                        // get the list
                        this.getResults(this);
                    } else {
                        this.toggleResults('hide');
                    }
                }

                nIterations++;
            }.bind(this), nIntervalMS);
        };

        Searchahead.prototype.stopInterval = function () {
            clearInterval(this.timeout);
            this.timeout = null;
        };

        Searchahead.prototype.restartInterval = function (nIntervalMS) {
            this.stopInterval();
            this.startInterval(nIntervalMS);
        };

        Mediator.subscribe('ui.init', function () {
            $('[data-searchahead]').each(function () {
                new Searchahead(this);
            });
        });

        return Searchahead;

    });
