define(['jquery', 'viewportService', 'mediator', 'raven'],
    function NavTabs($, viewportService, mediator,Raven) {
        'use strict';

        var component = {
            name: 'NavTabs',
            slug: 'navtab',
            channel: 'ui.navtab'
        };

        function NavTabs( selector ) {
            this.$selector = $(selector);

            if ( !( this instanceof NavTabs ) ) {
                return new NavTabs();
            }

            if ( !this.$selector.data( component.name ) ) {
                try {
                    this.init();
                } catch( err ) {
                    Raven.captureException( err, {
                        extra: {
                            module: component.name
                        }
                    });
                }
            }

        };



        NavTabs.prototype.init = function init() {
            this.$nav = $('.NavTabs');
            this.$toggle = this.$nav.find('.NavTabs__Toggle');
            this.$visible = this.$nav.find('.NavTabs__Visible');
            this.$hidden = this.$nav.find('.NavTabs__Hidden');
            this.classHidden = 'is-hidden';
            this.breaks = [];
            this.updateOnResize();
            mediator.subscribe('resize', $.proxy(this.updateOnResize,this), 'viewportService');
            this.$toggle.on('click', $.proxy(this.toggleDropdown, this));
        };

        NavTabs.prototype.toggleDropdown = function toggleDropdown() {
            this.$hidden.toggleClass(this.classHidden);
        };

        NavTabs.prototype.updateOnResize = function updateOnResize() {
            var spaceAvail = this.$toggle.hasClass(this.classHidden) ? this.$nav.outerWidth() : this.$nav.outerWidth() - this.$toggle.outerWidth(); // eslint-disable-line
            this.$hidden.attr('style', 'min-width:' + this.$toggle.outerWidth() + 'px');
            if (this.$visible.outerWidth() > spaceAvail) {
                this.breaks.push(this.$visible.outerWidth());
                this.$visible.children().last().prependTo(this.$hidden);

                if (this.$toggle.hasClass(this.classHidden)) {

                    this.$toggle.removeClass(this.classHidden);
                }
            } else {
                if (spaceAvail > this.breaks[this.breaks.length - 1]) {
                    this.$hidden.children().first().appendTo(this.$visible);
                    this.breaks.pop();
                }
                if (this.breaks.length < 1) {
                    this.$toggle.addClass(this.classHidden);
                    this.$hidden.addClass(this.classHidden);
                }
            }
            if (this.$visible.outerWidth() > spaceAvail) {
                this.updateOnResize();
            }
        };

        // Initialize ui.component on mediator event 'ui.init'
        mediator.subscribe( 'ui.init', function() {
            $( '[data-' + component.slug + ']' ).each( function() {
                new NavTabs( this );
            } );
        } );


        return NavTabs;
    }
);
