define( [ 'mediator', 'underscore', 'jquery' ],
    
    function( mediator, _, $ ) {

        var storedViewport,
            $win = $( window );

        function getDimension() {
            return { 
                width : document.documentElement.clientWidth, 
                height : document.documentElement.clientHeight
            };
        }

        // #### getScrollTop
        // Cross-browser implementation of getting the current scroll
        // offset
        // http://stackoverflow.com/questions/871399/cross-browser-method-for-detecting-the-scrolltop-of-the-browser-window
        function getScrollTop() {
            var scrollTop = window.pageYOffset || (
                document.documentElement.clientHeight ? 
                    document.documentElement.scrollTop : document.body.scrollTop );

            return scrollTop;
        }

        function getDocumentHeight() {
            var body = document.body,
                html = document.documentElement,
                height = Math.max( body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight );
        
            return height;
        }

        // #### storeCurrentViewport()
        //
        // This method computes the current viewport and stores it in
        // `storedViewport` for later use.  The method is triggered by
        // `broadcastViewport()` upon resizing or scrolling.
        function storeCurrentViewport() {
            storedViewport = {
                'scrollOffsetTop': getDimension().height + getScrollTop(),
                'viewportWidth': getDimension().width,
                'viewportHeight': getDimension().height,
                'viewportRange': testScreenSize()
            };
        }

        // #### inY( element, offset )
        // Returns true if element - offset is visible in the viewport
        function inY( element, offset ) {
            return ( element.offset().top - offset < storedViewport.scrollOffsetTop && element.is(':visible') );
        }

        // #### testScreenSize()
        //
        // Returns `large`, `medium` or `small`
        function testScreenSize() {
            var currentSize;

            if ( getDimension().width < 660 ) {
                currentSize = 'small';
            } else if ( getDimension().width >= 660 && getDimension().width < 1000 ) {
                currentSize = 'medium';
            } else {
                currentSize = 'large';
            }

            return currentSize;
        }

        // #### broadcastViewport()
        //
        // *Emitting* `viewportService:change` event, the payload is an object
        // containing the up-to-date viewport information.
        function broadcastViewport( event ) {
            var currentVersion = storedViewport.viewportRange;

            storeCurrentViewport();

            // broadcast on version change
            if ( currentVersion !== storedViewport.viewportRange ) {
                mediator.publish( 'changeRange', storedViewport, 'viewportService' );
            }

            // broadcast on general change
            mediator.publish( 'change', storedViewport, 'viewportService' );

            if ( event && event.type === 'scroll' || event.type === 'resize' ) {
                mediator.publish( event.type, storedViewport, 'viewportService' );
            }
        }

        // Initialize `storedViewport`.
        storeCurrentViewport();

        // On a `resize` or `scroll` event on the window, trigger the
        // broadcasting of the current viewport information.  The broadcast is
        // throttled and will fire every 150ms (max).
        $win
            .on( 'scroll.viewportService', _( broadcastViewport ).throttle( 150 ) )
            .on( 'resize.viewportService', _( broadcastViewport ).throttle( 150 ) );

        return {
            getViewport: function() {
                return storedViewport;
            },
            getScrollTop: function() {
                return getScrollTop;
            },
            getDocumentHeight: function() {
                return getDocumentHeight();
            },
            supports: function(size) {
                if ( !_( size ).isArray() ) {
                    size = [ size ];
                }
                return _( size ).contains( storedViewport.viewportRange );
            },
            inY: function( element, offset ) {
                return inY( element, offset );
            }
        };
    }
);